.jp-CellButton {
    position: absolute;
	right: 0;
	bottom: 0;
	top: 0;
	height: 100%;
	/* width: 34px; */
	padding: 9px 10px 0 0;
	display: block;
	z-index: 600;
	overflow: unset;
}

.cellButton {
    position: absolute;
    right: 188px;
	top: 10px;
    padding-right: 2px;
	background: none;
	border: none;
	padding: 0;
	cursor: pointer;
	padding-left: 2px;
	margin-bottom: 7px;
}

.cellButton-icon {
	display: flex;
    align-items: center;
    justify-content: center;
    padding: 3px;
    border-radius: 50%;
    box-shadow: 0 7.5px 20px rgba(51, 51, 51, 0.1);
	transition: 0.2s ease-in-out all;
	background-color: #cccccc;
}

.cellButton:hover .cellButton-icon {
	box-shadow: 0 7.5px 20px rgba(51, 51, 51, 0.5);
}